@@ -1,6 +1,7 @@
+/* -*- Mode:Text -*- */
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: tree.c,v 1.66 2005/04/14 14:43:46 geoff Exp $";
+    "$Id: tree.c,v 1.44 91/08/10 14:09:49 geoff Exp $";
 #endif
 
 /*
@@ -9,113 +10,183 @@ static char Rcs_Id[] =
  * Pace Willisson, 1983
  * Hash support added by Geoff Kuenning, 1987
  *
- * Copyright 1987-1989, 1992, 1993, 1999, 2001, 2005, Geoff Kuenning,
- * Claremont, CA
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All modifications to the source code must be clearly marked as
- *    such.  Binary redistributions based on modified source code
- *    must be clearly marked as modified versions in the documentation
- *    and/or other materials provided with the distribution.
- * 4. The code that causes the 'ispell -v' command to display a prominent
- *    link to the official ispell Web site may not be removed.
- * 5. The name of Geoff Kuenning may not be used to endorse or promote
- *    products derived from this software without specific prior
- *    written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY GEOFF KUENNING AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL GEOFF KUENNING OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
+ * Copyright 1987, 1988, 1989, by Geoff Kuenning, Manhattan Beach, CA
+ * Permission for non-profit use is hereby granted.
+ * All other rights reserved.
+ * See "version.h" for a more complete copyright notice.
  */
 
 /*
- * $Log: tree.c,v $
- * Revision 1.66  2005/04/14 14:43:46  geoff
- * Update copyright
- *
- * Revision 1.65  2005/04/14 14:38:23  geoff
- * Update license.  Don't segfault if HOME is unset in the environment
- * and PDICTHOME isn't set in config.X
- *
- * Revision 1.64  2001/09/06 00:30:29  geoff
- * Many changes from Eli Zaretskii to support DJGPP compilation.
- *
- * Revision 1.63  2001/07/25 21:51:46  geoff
- * Minor license update.
- *
- * Revision 1.62  2001/07/23 20:24:04  geoff
- * Update the copyright and the license.
- *
- * Revision 1.61  2000/08/22 10:52:25  geoff
- * Fix some compiler warnings.
- *
- * Revision 1.60  2000/06/26 20:43:50  geoff
- * Fix a minor lint bug.  If HOME is unset, still attempt to find the
- * personal dictionary using all ways that don't depend on HOME (in
- * particular, the -p switch).
- *
- * Revision 1.59  1999/01/18 03:28:45  geoff
- * Turn many char declarations into unsigned char, so that we won't have
- * sign-extension problems.
- *
- * Revision 1.58  1999/01/07  01:57:46  geoff
- * Update the copyright.
- *
- * Revision 1.57  1997/12/02  06:25:11  geoff
- * Get rid of some compile options that really shouldn't be optional.
- *
- * Revision 1.56  1995/01/08  23:23:49  geoff
- * Support PDICTHOME for DOS purposes.
- *
- * Revision 1.55  1994/10/25  05:46:27  geoff
- * Fix a comment that looked to some compilers like it might be nested.
- *
- * Revision 1.54  1994/01/25  07:12:15  geoff
- * Get rid of all old RCS log lines in preparation for the 3.1 release.
- *
+ * $Log:	tree.c,v $
+ * Revision 1.44  91/08/10  14:09:49  geoff
+ * Don't issue TREE_C_CANT_UPDATE if the personal dictionary doesn't
+ * exist at all.  Sleep for 2 seconds after issuing the message, so that
+ * the user has a chance to see it before ispell clears the screen.
+ * 
+ * Revision 1.43  91/07/27  20:48:39  geoff
+ * Major rewrite of the personal-dictionary initialization to support a
+ * personal dictionary in the local directory, and to merge it with the
+ * one from the home directory if possible.  Fix a compile error and a
+ * bug in the REGEX_LOOKUP code.
+ * 
+ * Revision 1.42  91/07/15  19:27:14  geoff
+ * Make tinsert static.  Mention that the argument to treeinsert must be
+ * canonical.  Provide the "canonical" parameter to all strtoichar,
+ * strtosichar, and casecmp calls.  Make sure the argument to casecmp is
+ * canonical.
+ * 
+ * Revision 1.41  91/07/11  19:52:29  geoff
+ * Remove the include of stdio.h, since ispell.h now does this.
+ * 
+ * Revision 1.40  91/07/05  20:32:00  geoff
+ * Fix some more lint complaints.
+ * 
+ * Revision 1.39  91/07/03  18:21:14  geoff
+ * Don't include types.h, dir.h, or param.h, since config.h now does that.
+ * 
+ * Revision 1.38  91/06/23  22:01:04  geoff
+ * On non-USG systems, include sys/dir.h for MAXNAMLEN
+ * 
+ * Revision 1.37  91/01/24  02:28:12  geoff
+ * Modify dictionary-finding code to be more consistent and predictable
+ * about where it looks, to create nonexistent dictionaries in $HOME, and
+ * to fix a bug that caused private dictionaries in the home to be clobbered.
+ * 
+ * Revision 1.36  90/12/31  00:59:34  geoff
+ * Reformat to follow a consistent convention throughout ispell
+ * 
+ * Revision 1.35  90/04/26  22:44:31  geoff
+ * Add the canonicalize parameter to the call to ichartostr.
+ * 
+ * Revision 1.34  89/12/27  03:19:02  geoff
+ * Move all messages to msgs.h so they can be reconfigured
+ * 
+ * Revision 1.33  89/07/11  00:25:41  geoff
+ * Add REGEX_LOOKUP support and Amiga support, based on that from
+ * luis@rice.edu.  Also change the name of the personal hash table to better
+ * distinguish it from the main one.
+ * 
+ * Revision 1.32  89/06/09  15:56:53  geoff
+ * Add support for the internal "character" type, ichar_t.
+ * 
+ * Revision 1.31  89/04/28  01:17:36  geoff
+ * Change Header to Id;  nobody cares about my pathnames.
+ * 
+ * Revision 1.30  89/04/03  01:59:15  geoff
+ * Fix a bunch of lint complaints.
+ * 
+ * Revision 1.29  88/12/26  02:33:53  geoff
+ * Add a copyright notice.
+ * 
+ * Revision 1.28  88/10/20  20:53:27  geoff
+ * Add support for getting a different personal dictionary when the -d
+ * switch is specified, for multilingual use of ispell.
+ * 
+ * Revision 1.27  88/04/30  22:16:25  geoff
+ * Fix some lint complaints.
+ * 
+ * Revision 1.26  88/03/27  01:06:03  geoff
+ * Fix a whole bunch of problems with the CAPITALIZATION option.
+ * 
+ * Revision 1.25  88/03/12  02:46:03  geoff
+ * Check the return status from makedent, and don't idiotically try to
+ * continue if it fails.
+ * 
+ * Revision 1.24  88/02/20  23:15:06  geoff
+ * Major changes to support the new capitalization handling, and to use
+ * the new subroutines in the "makedent.c" module.
+ * 
+ * Revision 1.23  87/09/30  23:32:04  geoff
+ * Move some globals to ispell.h.
+ * 
+ * Revision 1.22  87/09/26  15:53:07  geoff
+ * Make buffer sizes be more consistent.
+ * 
+ * Revision 1.21  87/09/24  23:24:35  geoff
+ * When changing a lowercase word to followcase, remember to set the
+ * capitalize flags.  (Bart Schaefer).
+ * 
+ * Revision 1.20  87/09/09  00:20:10  geoff
+ * Fix treeoutput() to be nondestructive (Doug Lind, Michael Wester).  It's
+ * faster too, since it now uses qsort.  Also fix several bugs in
+ * treeinsert:  capitalization information wasn't checked on memory
+ * overflow, and duplicate followcase entries weren't detected.
+ * 
+ * Revision 1.19  87/09/03  19:26:59  geoff
+ * Simplify the hash-size table now that we don't read entire dictionaries
+ * into the personal-hash tree.
+ * 
+ * Revision 1.18  87/07/20  23:23:27  geoff
+ * Modify to support the new flag format used with language tables.
+ * 
+ * Revision 1.17  87/06/07  15:04:25  geoff
+ * Don't refer to capitalization flags if CAPITALIZE is off (Gary Johnson)
+ * 
+ * Revision 1.16  87/05/27  23:28:40  geoff
+ * Reset hasslash in toutword so every word gets a slash
+ * 
+ * Revision 1.15  87/05/11  11:50:35  geoff
+ * Speed up SORTPERSONAL dramatically by not scanning unkept words repeatedly.
+ * 
+ * Revision 1.14  87/04/19  22:53:55  geoff
+ * Add SORTPERSONAL and capitalization support.
+ * 
+ * Revision 1.13  87/04/02  12:25:55  geoff
+ * Remove the obsolete treeprint() routine.  Put the "can't update
+ * personal dict" error message onto stderr, not stdout.
+ * 
+ * Revision 1.12  87/04/01  15:23:06  geoff
+ * Integrate Joe Orost's V7/register changes into the main branch
+ * 
+ * Revision 1.11  87/03/28  19:33:24  geoff
+ * Put the personal dictionary out in compressed (no extra slashes) form
+ * 
+ * Revision 1.10  87/03/27  17:21:48  geoff
+ * Accept (but don't require) new-format dictionaries without extra slashes.
+ * However, don't generate them on dumps yet.
+ * 
+ * Revision 1.9  87/03/22  23:28:03  geoff
+ * Integrate Perry Smith's changes into the main branch
+ * 
+ * Revision 1.8  87/03/12  23:36:54  geoff
+ * Fix treeprint, which didn't catch up to the latest changes.  Remove the
+ * list-link following in the main hash table in treeoutput, since the
+ * entries will eventually be found by the linear search.
+ * 
+ * Revision 1.7  87/03/10  23:33:13  geoff
+ * Add code to deal gracefully with memory allocation failures.  If small
+ * allocations fail, put out a message and exit.  If a big hash table allocation
+ * fails, put out a message, but continue anyway, just filling the existing
+ * table overfull.
+ * 
+ * Revision 1.6  87/03/08  20:31:27  geoff
+ * Improve the hash sizes table to be appropriate to really big dictionaries.
+ * Change the personal-dictionary opening code to be more flexible about
+ * where it looks, so that relative pathnames will work sensibly.
+ * Make lots of changes in the hashing so it works better and faster.
+ * 
+ * Revision 1.5  87/03/01  00:57:17  geoff
+ * Major changes to use a hash table instead of a binary tree, allow
+ * user dictionaries to have suffixes, and integrate the user dictionary
+ * with the main one.
+ * 
+ * Revision 1.4  87/02/28  14:58:44  geoff
+ * Modify to support suffix flags in the user dictionary.
+ * 
+ * Revision 1.3  87/02/26  00:25:23  geoff
+ * Integrate McQueer's enhancements into the main branch
+ * 
+ * Revision 1.2  87/01/17  13:12:28  geoff
+ * Add RCS ID keywords
+ * 
  */
 
 #include <ctype.h>
 #include <errno.h>
 #include "config.h"
 #include "ispell.h"
-#include "proto.h"
 #include "msgs.h"
 
-void		treeinit P ((char * p, char * LibDict));
-static FILE *	trydict P ((char * dictname, char * home,
-		  char * prefix, char * suffix));
-static void	treeload P ((FILE * dictf));
-void		treeinsert P ((unsigned char * word, int wordlen, int keep));
-static struct dent * tinsert P ((struct dent * proto));
-struct dent *	treelookup P ((ichar_t * word));
-#if SORTPERSONAL != 0
-static int	pdictcmp P ((struct dent ** enta, struct dent **entb));
-#endif /* SORTPERSONAL != 0 */
-void		treeoutput P ((void));
-VOID *		mymalloc P ((unsigned int size));
-void		myfree P ((VOID * ptr));
-#ifdef REGEX_LOOKUP
-char *		do_regex_lookup P ((char * expr, int whence));
-#endif /* REGEX_LOOKUP */
-
 static int		cantexpand = 0;	/* NZ if an expansion fails */
 static struct dent *	pershtab;	/* Aux hash table for personal dict */
 static int		pershsize = 0;	/* Space available in aux hash table */
@@ -132,18 +203,40 @@ static int goodsizes[] =
     53, 223, 907, 3631
     };
 
+static void		treeload ();
+void			treeinsert ();
+static struct dent *	tinsert ();
+struct dent *		treelookup ();
+
 static char		personaldict[MAXPATHLEN];
 static FILE *		dictf;
-static int		newwords = 0;
+static			newwords = 0;
 
-void treeinit (p, LibDict)
+extern struct dent *	lookup();
+extern void		upcase();
+
+void			myfree ();
+
+#ifndef linux
+extern char *		index ();
+extern char *		calloc ();
+extern char *		malloc ();
+extern char *		realloc ();
+extern void		free ();
+extern void		exit ();
+extern void		perror ();
+extern char *		getenv();
+extern char *		strcpy ();
+extern void		qsort ();
+#endif
+
+treeinit (p, LibDict)
     char *		p;		/* Value specified in -p switch */
     char *		LibDict;	/* Root of default dict name */
     {
     int			abspath;	/* NZ if p is abs path name */
     char *		h;		/* Home directory name */
     char		seconddict[MAXPATHLEN]; /* Name of secondary dict */
-    FILE *		secondf;	/* Access to second dict file */
 
     /*
     ** If -p was not specified, try to get a default name from the
@@ -155,17 +248,15 @@ void treeinit (p, LibDict)
     if (p == NULL)
 	p = getenv (PDICTVAR);
     /*
-    ** Figure out the user's home.  If HOME is unset (which can happen
-    ** if ispell is invoked unusually, such as from a daemon), we
-    ** won't look for any HOME personal dictionaries.  Exception:  If
-    ** PDICTHOME is set, we use it as a default for HOME.
+    ** if p exists and begins with '/' we don't really need HOME,
+    ** but it's not very likely that HOME isn't set anyway.
     */
-    if ((h = getenv (HOME)) == NULL)
-	{
-#ifdef PDICTHOME
-	h = PDICTHOME;
-#endif /* PDICTHOME */
-	}
+    if ((h = getenv ("HOME")) == NULL)
+#ifdef AMIGA
+	h = LIBDIR;
+#else /* AMIGA */
+	return;
+#endif /* AMIGA */
 
     if (p == NULL)
 	{
@@ -174,60 +265,53 @@ void treeinit (p, LibDict)
 	 * figure out the name of the personal dictionary and where it
 	 * is.  The rules are as follows:
 	 *
-	 * (1) If there is a local dictionary and a HOME dictionary,
-	 *     both are loaded, but changes are saved in the local one.
-	 *     The dictionary to save changes in is named "personaldict".
-	 * (2) Dictionaries named after the affix file take precedence
-	 *     over dictionaries with the default suffix (DEFPAFF).
-	 * (3) Dictionaries named with the new default names
-	 *     (DEFPDICT/DEFPAFF) take precedence over the old ones
-	 *     (OLDPDICT/OLDPAFF).
-	 * (4) Dictionaries aren't combined unless they follow the same
-	 *     naming scheme.
-	 * (5) If no dictionary can be found, a new one is created in
-	 *     the home directory, named after DEFPDICT and the affix
-	 *     file.
+	 * (1) If there is a dictionary in the local directory which
+	 *     is named after the hash file, this will become
+	 *     "personaldict", which is where any changes will be saved.
+	 * (2) If there is also a dictionary in $HOME, we will load
+	 *     it, regardless of whether the dictionary listed in (1)
+	 *     exists.  If step (1) failed, this will also become
+	 *     "personaldict".
+	 * (3) If both previous steps fail, we will try them again,
+	 *     using DEFPAFF as the suffix.
 	 */
-	dictf = trydict (personaldict, (char *) NULL, DEFPDICT, LibDict);
-	if (h == NULL)
-	    secondf = NULL;
+	(void) sprintf (personaldict, "%s%s", DEFPDICT, LibDict);
+	(void) sprintf (seconddict, "%s/%s%s", h, DEFPDICT, LibDict);
+	if ((dictf = fopen (personaldict, "r")) == NULL)
+	    personaldict[0] = '\0';
 	else
-	    secondf = trydict (seconddict, h, DEFPDICT, LibDict);
-	if (dictf == NULL  &&  secondf == NULL)
 	    {
-	    dictf = trydict (personaldict, (char *) NULL, DEFPDICT, DEFPAFF);
-	    if (h != NULL)
-		secondf = trydict (seconddict, h, DEFPDICT, DEFPAFF);
+	    treeload (dictf);
+	    (void) fclose (dictf);
 	    }
-	if (dictf == NULL  &&  secondf == NULL)
+	if ((dictf = fopen (seconddict, "r")) == NULL)
+	    seconddict[0] = '\0';
+	else
 	    {
-	    dictf = trydict (personaldict, (char *) NULL, OLDPDICT, LibDict);
-	    if (h != NULL)
-		secondf = trydict (seconddict, h, OLDPDICT, LibDict);
-	    }
-	if (dictf == NULL  &&  secondf == NULL)
-	    {
-	    dictf = trydict (personaldict, (char *) NULL, OLDPDICT, OLDPAFF);
-	    if (h != NULL)
-		secondf = trydict (seconddict, h, OLDPDICT, OLDPAFF);
+	    treeload (dictf);
+	    (void) fclose (dictf);
 	    }
 	if (personaldict[0] == '\0')
 	    {
 	    if (seconddict[0] != '\0')
 		(void) strcpy (personaldict, seconddict);
 	    else
-		(void) sprintf (personaldict, "%s/%s%s", h == NULL ? "" : h,
-		  DEFPDICT, LibDict);
-	    }
-	if (dictf != NULL)
-	    {
-	    treeload (dictf);
-	    (void) fclose (dictf);
-	    }
-	if (secondf != NULL)
-	    {
-	    treeload (secondf);
-	    (void) fclose (secondf);
+		{
+		(void) sprintf (personaldict, "%s%s", DEFPDICT, DEFPAFF);
+		(void) sprintf (seconddict, "%s/%s%s", h, DEFPDICT, DEFPAFF);
+		if ((dictf = fopen (personaldict, "r")) == NULL)
+		    (void) strcpy (personaldict, seconddict);
+		else
+		    {
+		    treeload (dictf);
+		    (void) fclose (dictf);
+		    }
+		if ((dictf = fopen (seconddict, "r")) != NULL)
+		    {
+		    treeload (dictf);
+		    (void) fclose (dictf);
+		    }
+		}
 	    }
 	}
     else
@@ -237,16 +321,8 @@ void treeinit (p, LibDict)
 	** with "./" and "../" is considered an absolute path, since this
 	** still means we can't prepend HOME.
 	*/
-	abspath = IS_SLASH (*p)  ||  strncmp (p, "./", 2) == 0
-	  ||  strncmp (p, "../", 3) == 0;
-#ifdef MSDOS
-	/*
-	** DTRT with drive-letter braindamage and with backslashes.
-	*/
-	abspath |= strncmp (p, ".\\", 2) == 0
-	  || strncmp (p, "..\\", 3) == 0
-	  || (p[0] != '\0'  &&  p[1] == ':');
-#endif
+	abspath = (*p == '/'  ||  strncmp (p, "./", 2) == 0
+	  ||  strncmp (p, "../", 3) == 0);
 	if (abspath)
 	    {
 	    (void) strcpy (personaldict, p);
@@ -262,9 +338,7 @@ void treeinit (p, LibDict)
 	    ** The user gave us a relative pathname.  We will try it
 	    ** locally, and if that doesn't work, we'll try the home
 	    ** directory.  If neither exists, it will be created in
-	    ** the home directory if words are added, unless HOME was
-	    ** unset, in which case the relative pathname will be used
-	    ** as a creation point.
+	    ** the home directory if words are added.
 	    */
 	    (void) strcpy (personaldict, p);
 	    if ((dictf = fopen (personaldict, "r")) != NULL)
@@ -272,7 +346,7 @@ void treeinit (p, LibDict)
 		treeload (dictf);
 		(void) fclose (dictf);
 		}
-	    else if (!abspath  &&  h != NULL)
+	    else if (!abspath)
 		{
 		/* Try the home */
 		(void) sprintf (personaldict, "%s/%s", h, p);
@@ -288,7 +362,7 @@ void treeinit (p, LibDict)
 	     */
 	    if (dictf == NULL)
 		{
-		(void) fprintf (stderr, CANT_OPEN, p, MAYBE_CR (stderr));
+		(void) fprintf (stderr, CANT_OPEN, p);
 		perror ("");
 		return;
 		}
@@ -296,61 +370,40 @@ void treeinit (p, LibDict)
 	}
 
     if (!lflag  &&  !aflag
-      &&  access (personaldict, W_OK) < 0  &&  errno != ENOENT)
+      &&  access (personaldict, 2) < 0  &&  errno != ENOENT)
 	{
-	(void) fprintf (stderr, TREE_C_CANT_UPDATE, personaldict,
-	  MAYBE_CR (stderr));
-	(void) sleep ((unsigned) 2);
+	(void) fprintf (stderr, TREE_C_CANT_UPDATE, personaldict);
+	(void) sleep (2);
 	}
     }
 
-/*
- * Try to open a dictionary.  As a side effect, leaves the dictionary
- * name in "filename" if one is found, and leaves a null string there
- * otherwise.
- */
-static FILE * trydict (filename, home, prefix, suffix)
-    char *		filename;	/* Where to store the file name */
-    char *		home;		/* Home directory */
-    char *		prefix;		/* Prefix for dictionary */
-    char *		suffix;		/* Suffix for dictionary */
-    {
-    FILE *		dictf;		/* Access to dictionary file */
-
-    if (home == NULL)
-	(void) sprintf (filename, "%s%s", prefix, suffix);
-    else
-	(void) sprintf (filename, "%s/%s%s", home, prefix, suffix);
-    dictf = fopen (filename, "r");
-    if (dictf == NULL)
-	filename[0] = '\0';
-    return dictf;
-    }
-
-static void treeload (loadfile)
-    register FILE *	loadfile;	/* File to load words from */
+static void treeload (dictf)
+    register FILE *	dictf;		/* File to load words from */
     {
     char		buf[BUFSIZ];	/* Buffer for reading pers dict */
 
-    while (fgets (buf, sizeof buf, loadfile) != NULL)
-	treeinsert ((unsigned char *) buf, sizeof buf, 1);
+    while (fgets (buf, sizeof buf, dictf) != NULL)
+	treeinsert (buf, 1);
     newwords = 0;
     }
 
-void treeinsert (word, wordlen, keep)
-    unsigned char *	word;	/* Word to insert - must be canonical */
-    int			wordlen; /* Length of the word buffer */
+void treeinsert (word, keep)
+    char *		word;	/* Word to insert - must be canonical */
     int			keep;
     {
-    register unsigned int i;
+    register int	i;
     struct dent		wordent;
     register struct dent * dp;
     struct dent *	olddp;
+#ifdef CAPITALIZATION
     struct dent *	newdp;
+#endif
     struct dent *	oldhtab;
-    unsigned int	oldhsize;
+    int			oldhsize;
     ichar_t		nword[INPUTWORDLEN + MAXAFFIXLEN];
+#ifdef CAPITALIZATION
     int			isvariant;
+#endif
 
     /*
      * Expand hash table when it is MAXPCT % full.
@@ -372,7 +425,7 @@ void treeinsert (word, wordlen, keep)
 	  (struct dent *) calloc ((unsigned) pershsize, sizeof (struct dent));
 	if (pershtab == NULL)
 	    {
-	    (void) fprintf (stderr, TREE_C_NO_SPACE, MAYBE_CR (stderr));
+	    (void) fprintf (stderr, TREE_C_NO_SPACE);
 	    /*
 	     * Try to continue anyway, since our overflow
 	     * algorithm can handle an overfull (100%+) table,
@@ -382,7 +435,7 @@ void treeinsert (word, wordlen, keep)
 	     */
 	    if (oldhtab == NULL)
 		exit (1);		/* No old table, can't go on */
-	    (void) fprintf (stderr, TREE_C_TRY_ANYWAY, MAYBE_CR (stderr));
+	    (void) fprintf (stderr, TREE_C_TRY_ANYWAY);
 	    cantexpand = 1;		/* Suppress further messages */
 	    pershsize = oldhsize;	/* Put things back */
 	    pershtab = oldhtab;		/* ... */
@@ -398,9 +451,14 @@ void treeinsert (word, wordlen, keep)
 		dp = &oldhtab[i];
 		if (dp->flagfield & USED)
 		    {
+#ifdef CAPITALIZATION
 		    newdp = tinsert (dp);
 		    isvariant = (dp->flagfield & MOREVARIANTS);
+#else
+		    (void) tinsert (dp);
+#endif
 		    dp = dp->next;
+#ifdef CAPITALIZATION
 		    while (dp != NULL)
 			{
 			if (isvariant)
@@ -421,6 +479,15 @@ void treeinsert (word, wordlen, keep)
 			    free ((char *) olddp);
 			    }
 			}
+#else
+		    while (dp != NULL)
+			{
+			(void) tinsert (dp);
+			olddp = dp;
+			dp = dp->next;
+			free ((char *) olddp);
+			}
+#endif
 		    }
 		}
 	    if (oldhtab != NULL)
@@ -432,7 +499,7 @@ void treeinsert (word, wordlen, keep)
     ** We're ready to do the insertion.  Start by creating a sample
     ** entry for the word.
     */
-    if (makedent (word, wordlen, &wordent) < 0)
+    if (makedent (word, &wordent) < 0)
 	return;			/* Word must be too big or something */
     if (keep)
 	wordent.flagfield |= KEEP;
@@ -440,7 +507,7 @@ void treeinsert (word, wordlen, keep)
     ** Now see if word or a variant is already in the table.  We use the
     ** capitalized version so we'll find the header, if any.
     **/
-    (void) strtoichar (nword, word, sizeof nword, 1);
+    strtoichar (nword, word, 1);
     upcase (nword);
     if ((dp = lookup (nword, 1)) != NULL)
 	{
@@ -455,8 +522,10 @@ void treeinsert (word, wordlen, keep)
 	{
 	/* It's new. Insert the word. */
 	dp = tinsert (&wordent);
+#ifdef CAPITALIZATION
 	if (captype (dp->flagfield) == FOLLOWCASE)
 	   (void) addvheader (dp);
+#endif
 	}
     newwords |= keep;
     }
@@ -469,8 +538,10 @@ static struct dent * tinsert (proto)
     register struct dent * hp;		/* Next trial entry in hash table */
     register struct dent * php;		/* Prev. value of hp, for chaining */
 
-    if (strtoichar (iword, proto->word, sizeof iword, 1))
-	(void) fprintf (stderr, WORD_TOO_LONG ((char *) proto->word));
+    strtoichar (iword, proto->word, 1);
+#ifndef CAPITALIZATION
+    upcase (iword);
+#endif
     hcode = hash (iword, pershsize);
     php = NULL;
     hp = &pershtab[hcode];
@@ -484,7 +555,7 @@ static struct dent * tinsert (proto)
 	hp = (struct dent *) calloc (1, sizeof (struct dent));
 	if (hp == NULL)
 	    {
-	    (void) fprintf (stderr, TREE_C_NO_SPACE, MAYBE_CR (stderr));
+	    (void) fprintf (stderr, TREE_C_NO_SPACE);
 	    exit (1);
 	    }
 	}
@@ -504,15 +575,17 @@ struct dent * treelookup (word)
 
     if (pershsize <= 0)
 	return NULL;
-    (void) ichartostr ((unsigned char *) chword, word, sizeof chword, 1);
+    (void) ichartostr (chword, word, 1);
     hcode = hash (word, pershsize);
     hp = &pershtab[hcode];
     while (hp != NULL  &&  (hp->flagfield & USED))
 	{
-	if (strcmp (chword, (char *) hp->word) == 0)
+	if (strcmp (chword, hp->word) == 0)
 	    break;
+#ifdef CAPITALIZATION
 	while (hp->flagfield & MOREVARIANTS)
 	    hp = hp->next;
+#endif
 	hp = hp->next;
 	}
     if (hp != NULL  &&  (hp->flagfield & USED))
@@ -523,12 +596,12 @@ struct dent * treelookup (word)
 
 #if SORTPERSONAL != 0
 /* Comparison routine for sorting the personal dictionary with qsort */
-static int pdictcmp (enta, entb)
+pdictcmp (enta, entb)
     struct dent **	enta;
     struct dent **	entb;
     {
 
-    /* The parentheses around *enta and *entb below are NECESSARY!
+    /* The parentheses around *enta/*entb below are NECESSARY!
     ** Otherwise the compiler reads it as *(enta->word), or
     ** enta->word[0], which is illegal (but pcc takes it and
     ** produces wrong code).
@@ -537,7 +610,7 @@ static int pdictcmp (enta, entb)
     }
 #endif
 
-void treeoutput ()
+treeoutput ()
     {
     register struct dent *	cent;	/* Current entry */
     register struct dent *	lent;	/* Linked entry */
@@ -553,7 +626,7 @@ void treeoutput ()
 
     if ((dictf = fopen (personaldict, "w")) == NULL)
 	{
-	(void) fprintf (stderr, CANT_CREATE, personaldict, MAYBE_CR (stderr));
+	(void) fprintf (stderr, CANT_CREATE, personaldict);
 	return;
 	}
 
@@ -562,11 +635,11 @@ void treeoutput ()
     ** If we are going to sort the personal dictionary, we must know
     ** how many items are going to be sorted.
     */
-    pdictsize = 0;
     if (hcount >= SORTPERSONAL)
 	sortlist = NULL;
     else
 	{
+	pdictsize = 0;
 	for (cent = pershtab, ehtab = pershtab + pershsize;
 	  cent < ehtab;
 	  cent++)
@@ -575,8 +648,10 @@ void treeoutput ()
 		{
 		if ((lent->flagfield & (USED | KEEP)) == (USED | KEEP))
 		    pdictsize++;
+#ifdef CAPITALIZATION
 		while (lent->flagfield & MOREVARIANTS)
 		  lent = lent->next;
+#endif
 		}
 	    }
 	for (cent = hashtbl, ehtab = hashtbl + hashsize;
@@ -592,8 +667,10 @@ void treeoutput ()
 		** test below.  This test will appear
 		** several more times in this routine.
 		*/
+#ifdef CAPITALIZATION
 		if (captype (cent->flagfield) != FOLLOWCASE
 		  &&  cent->word != NULL)
+#endif
 		    pdictsize++;
 		}
 	    }
@@ -611,8 +688,10 @@ void treeoutput ()
 		if ((lent->flagfield & (USED | KEEP)) == (USED | KEEP))
 		    {
 		    toutent (dictf, lent, 1);
+#ifdef CAPITALIZATION
 		    while (lent->flagfield & MOREVARIANTS)
 			lent = lent->next;
+#endif
 		    }
 		}
 	    }
@@ -622,8 +701,10 @@ void treeoutput ()
 	    {
 	    if ((cent->flagfield & (USED | KEEP)) == (USED | KEEP))
 		{
+#ifdef CAPITALIZATION
 		if (captype (cent->flagfield) != FOLLOWCASE
 		  &&  cent->word != NULL)
+#endif
 		    toutent (dictf, cent, 1);
 		}
 	    }
@@ -645,8 +726,10 @@ void treeoutput ()
 	    if ((lent->flagfield & (USED | KEEP)) == (USED | KEEP))
 		{
 		*sortptr++ = lent;
+#ifdef CAPITALIZATION
 		while (lent->flagfield & MOREVARIANTS)
 		    lent = lent->next;
+#endif
 		}
 	    }
 	}
@@ -654,15 +737,16 @@ void treeoutput ()
 	{
 	if ((cent->flagfield & (USED | KEEP)) == (USED | KEEP))
 	    {
+#ifdef CAPITALIZATION
 	    if (captype (cent->flagfield) != FOLLOWCASE
 	      &&  cent->word != NULL)
+#endif
 		*sortptr++ = cent;
 	    }
 	}
     /* Sort the list */
     qsort ((char *) sortlist, (unsigned) pdictsize,
-      sizeof (sortlist[0]),
-      (int (*) P ((const void *, const void *))) pdictcmp);
+      sizeof (sortlist[0]), pdictcmp);
     /* Write it out */
     for (sortptr = sortlist;  --pdictsize >= 0;  )
 	toutent (dictf, *sortptr++, 1);
@@ -674,18 +758,17 @@ void treeoutput ()
     (void) fclose (dictf);
     }
 
-VOID * mymalloc (size)
-    unsigned int	size;
+char * mymalloc (size)
     {
 
     return malloc ((unsigned) size);
     }
 
 void myfree (ptr)
-    VOID * ptr;
+    char * ptr;
     {
-    if (hashstrings != NULL  &&  (unsigned char *) ptr >= hashstrings
-      &&  (unsigned char *) ptr <= hashstrings + hashheader.stringsize)
+    if (hashstrings != NULL  &&  ptr >= hashstrings
+      &&  ptr <= hashstrings + hashheader.stringsize)
 	return;			/* Can't free stuff in hashstrings */
     free (ptr);
     }
@@ -703,7 +786,7 @@ char * do_regex_lookup (expr, whence)
     static int		    curindex;
     static struct dent *    curpersent;
     static int		    curpersindex;
-    static REGCTYPE	    cmp_expr = (REGCTYPE) NULL;
+    static char *	    cmp_expr;
     char		    dummy[INPUTWORDLEN + MAXAFFIXLEN];
     ichar_t *		    is;
 
@@ -712,8 +795,7 @@ char * do_regex_lookup (expr, whence)
 	is = strtosichar (expr, 0);
 	upcase (is);
 	expr = ichartosstr (is, 1);
-	REGFREE (cmp_expr);	/* free previous compiled pattern, if any */
-        cmp_expr = REGCMP (cmp_expr, expr);
+        cmp_expr = REGCMP (expr);
         curent = hashtbl;
         curindex = 0;
         curpersent = pershtab;
@@ -724,19 +806,19 @@ char * do_regex_lookup (expr, whence)
     for (  ; curindex < hashsize;  curent++, curindex++)
 	{
         if (curent->word != NULL
-          &&  REGEX (cmp_expr, (char *) curent->word, dummy) != NULL)
+          &&  REGEX (cmp_expr, curent->word, dummy) != NULL)
 	    {
 	    curindex++;
 	    /* Everybody's gotta write a wierd expression once in a while! */
-	    return (char *) curent++->word;
+	    return curent++->word;
 	    }
 	}
     /* Try the personal dictionary too */
     for (  ; curpersindex < pershsize;  curpersent++, curpersindex++)
 	{
         if ((curpersent->flagfield & USED) != 0
-          &&  curpersent->word != NULL
-          &&  REGEX (cmp_expr, (char *) curpersent->word, dummy) != NULL)
+          &  curpersent->word != NULL
+          &&  REGEX (cmp_expr, curpersent->word, dummy) != NULL)
 	    {
 	    curpersindex++;
 	    /* Everybody's gotta write a wierd expression once in a while! */
@@ -745,4 +827,5 @@ char * do_regex_lookup (expr, whence)
 	}
     return NULL;
     }
+
 #endif /* REGEX_LOOKUP */
